@@ -13,18 +13,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-from typing import Dict, List, Optional, Union
+from typing import Dict, List, Union
 
 import cloudpickle
 import mlflow
 import pandas as pd
 import prophet
 
+from mlflow.models.signature import ModelSignature
+
 from databricks.automl_runtime.forecast import OFFSET_ALIAS_MAP
-from mlflow.exceptions import MlflowException
-from mlflow.models.signature import infer_signature, ModelSignature
-from mlflow.protos.databricks_pb2 import INVALID_PARAMETER_VALUE
-from databricks.automl_runtime import version
+from databricks.automl_runtime.forecast.model import ForecastModel, mlflow_forecast_log_model
 
 
 PROPHET_CONDA_ENV = {
@@ -34,7 +33,7 @@ PROPHET_CONDA_ENV = {
             "pip": [
                 f"prophet=={prophet.__version__}",
                 f"cloudpickle=={cloudpickle.__version__}",
-                f"databricks-automl-runtime=={version.__version__}",
+                f"databricks-automl-runtime==0.2.5",
             ]
         }
     ],
@@ -42,7 +41,7 @@ PROPHET_CONDA_ENV = {
 }
 
 
-class ProphetModel(mlflow.pyfunc.PythonModel):
+class ProphetModel(ForecastModel):
     """
     Prophet mlflow model wrapper for univariate forecasting.
     """
@@ -63,17 +62,6 @@ class ProphetModel(mlflow.pyfunc.PythonModel):
         self._time_col = time_col
         super().__init__()
 
-    def _validate_cols(self, df: pd.DataFrame, required_cols: List[str]):
-        df_cols = set(df.columns)
-        required_cols_set = set(required_cols)
-        if not required_cols_set.issubset(df_cols):
-            raise MlflowException(
-                message=(
-                    f"Input data columns '{list(df_cols)}' do not contain the required columns '{required_cols}'"
-                ),
-                error_code=INVALID_PARAMETER_VALUE,
-            )
-
     def load_context(self, context: mlflow.pyfunc.model.PythonModelContext) -> None:
         """
         Loads artifacts from the specified :class:`~PythonModelContext` that can be used
@@ -83,6 +71,10 @@ class ProphetModel(mlflow.pyfunc.PythonModel):
         from prophet import Prophet  # noqa: F401
         return
 
+    @property
+    def model_env(self):
+        return PROPHET_CONDA_ENV
+
     def model(self) -> prophet.forecaster.Prophet:
         """
         Deserialize a Prophet model from json string
@@ -134,8 +126,7 @@ class ProphetModel(mlflow.pyfunc.PythonModel):
         if sample_input is None:
             sample_input = self._make_future_dataframe(horizon=1)
             sample_input.rename(columns={"ds": self._time_col}, inplace=True)
-        signature = infer_signature(sample_input, self.predict(context=None, model_input=sample_input))
-        return signature
+        return super().infer_signature(sample_input)
 
 
 class MultiSeriesProphetModel(ProphetModel):
@@ -162,16 +153,14 @@ class MultiSeriesProphetModel(ProphetModel):
         self._timeseries_starts = timeseries_starts
         self._id_cols = id_cols
 
-    def model(self, id: str) -> Optional[prophet.forecaster.Prophet]:
+    def model(self, id: str) -> prophet.forecaster.Prophet:
         """
         Deserialize one Prophet model from json string based on the id
         :param id: identity for the Prophet model
         :return: Prophet model
         """
         from prophet.serialize import model_from_json
-        if id in self._model_json:
-            return model_from_json(self._model_json[id])
-        return None
+        return model_from_json(self._model_json[id])
 
     def _make_future_dataframe(self, id: str, horizon: int) -> pd.DataFrame:
         """
@@ -254,12 +243,7 @@ class MultiSeriesProphetModel(ProphetModel):
         test_df = model_input.copy()
         test_df["ts_id"] = test_df[self._id_cols].astype(str).agg('-'.join, axis=1)
         test_df.rename(columns={self._time_col: "ds"}, inplace=True)
-
-        def model_prediction(df):
-            model = self.model(df.name)
-            if model:
-                return model.predict(df)
-        predict_df = test_df.groupby("ts_id").apply(lambda df: model_prediction(df)).reset_index()
+        predict_df = test_df.groupby("ts_id").apply(lambda df: self.model(df.name[0]).predict(df)).reset_index()
         return_df = test_df.merge(predict_df, how="left", on=["ts_id", "ds"])
         return return_df["yhat"]
 
@@ -271,9 +255,4 @@ def mlflow_prophet_log_model(prophet_model: Union[ProphetModel, MultiSeriesProph
     :param prophet_model: Prophet model wrapper
     :param sample_input: sample input Dataframes for model inference
     """
-    # log the model without signature if infer_signature is failed.
-    try:
-        signature = prophet_model.infer_signature(sample_input)
-    except Exception: # noqa
-        signature = None
-    mlflow.pyfunc.log_model("model", conda_env=PROPHET_CONDA_ENV, python_model=prophet_model, signature=signature)
+    mlflow_forecast_log_model(prophet_model, sample_input)
